@@ -1,22 +1,17 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import sys
 import warnings
-from collections import defaultdict
-from random import choice as random_choice
+from collections import abc, defaultdict
+from random import shuffle
 from typing import (
     Any,
     Callable,
@@ -26,6 +21,7 @@ from typing import (
     Sequence,
     TypeVar,
     Union,
+    cast,
     overload,
 )
 
@@ -46,7 +42,10 @@ from hypothesis.internal.conjecture.utils import (
 )
 from hypothesis.internal.coverage import check_function
 from hypothesis.internal.lazyformat import lazyformat
-from hypothesis.internal.reflection import get_pretty_function_description
+from hypothesis.internal.reflection import (
+    get_pretty_function_description,
+    is_identity_function,
+)
 from hypothesis.strategies._internal.utils import defines_strategy
 from hypothesis.utils.conventions import UniqueIdentifier
 
@@ -182,7 +181,7 @@ def recursive_property(name, default):
             # Note: This is actually covered, by test_very_deep_deferral
             # in tests/cover/test_deferred_strategies.py. Unfortunately it
             # runs into a coverage bug. See
-            # https://bitbucket.org/ned/coveragepy/issues/605/
+            # https://github.com/nedbat/coveragepy/issues/605
             # for details.
             if count > 50:  # pragma: no cover
                 key = frozenset(mapping.items())
@@ -289,6 +288,7 @@ class SearchStrategy(Generic[Ex]):
                 "it performs better, saves and replays failures to avoid flakiness, "
                 "and reports minimal examples. (strategy: %r)" % (self,),
                 NonInteractiveExampleWarning,
+                stacklevel=2,
             )
 
         context = _current_build_context.value
@@ -312,6 +312,11 @@ class SearchStrategy(Generic[Ex]):
                     "#drawing-interactively-in-tests for more details."
                 )
 
+        try:
+            return self.__examples.pop()
+        except (AttributeError, IndexError):
+            self.__examples: List[Ex] = []
+
         from hypothesis.core import given
 
         # Note: this function has a weird name because it might appear in
@@ -319,18 +324,18 @@ class SearchStrategy(Generic[Ex]):
         @given(self)
         @settings(
             database=None,
-            max_examples=10,
+            max_examples=100,
             deadline=None,
             verbosity=Verbosity.quiet,
             phases=(Phase.generate,),
             suppress_health_check=HealthCheck.all(),
         )
         def example_generating_inner_function(ex):
-            examples.append(ex)
+            self.__examples.append(ex)
 
-        examples: List[Ex] = []
         example_generating_inner_function()
-        return random_choice(examples)
+        shuffle(self.__examples)
+        return self.__examples.pop()
 
     def map(self, pack: Callable[[Ex], T]) -> "SearchStrategy[T]":
         """Returns a new strategy that generates values by generating a value
@@ -338,6 +343,8 @@ class SearchStrategy(Generic[Ex]):
 
         This method is part of the public API.
         """
+        if is_identity_function(pack):
+            return self  # type: ignore  # Mypy has no way to know that `Ex == T`
         return MappedSearchStrategy(pack=pack, strategy=self)
 
     def flatmap(
@@ -419,13 +426,13 @@ class SearchStrategy(Generic[Ex]):
         return result
 
     @property
-    def label(self):
+    def label(self) -> int:
         if self.__label is calculating:
             return 0
         if self.__label is None:
             self.__label = calculating
             self.__label = self.calc_label()
-        return self.__label
+        return cast(int, self.__label)
 
     def calc_label(self):
         return self.class_label
@@ -458,7 +465,7 @@ class SampledFromStrategy(SearchStrategy):
     """
 
     def __init__(self, elements, repr_=None, transformations=()):
-        SearchStrategy.__init__(self)
+        super().__init__()
         self.elements = cu.check_sample(elements, "sampled_from")
         assert self.elements
         self.repr_ = repr_
@@ -592,7 +599,7 @@ class OneOfStrategy(SearchStrategy):
     """
 
     def __init__(self, strategies):
-        SearchStrategy.__init__(self)
+        super().__init__()
         strategies = tuple(strategies)
         self.original_strategies = list(strategies)
         self.__element_strategies = None
@@ -676,57 +683,61 @@ class OneOfStrategy(SearchStrategy):
 
 
 @overload
-def one_of(args: Sequence[SearchStrategy[Any]]) -> SearchStrategy[Any]:
-    raise NotImplementedError
+def one_of(
+    __args: Sequence[SearchStrategy[Any]],
+) -> SearchStrategy[Any]:  # pragma: no cover
+    ...
 
 
 @overload  # noqa: F811
-def one_of(a1: SearchStrategy[Ex]) -> SearchStrategy[Ex]:
-    raise NotImplementedError
+def one_of(__a1: SearchStrategy[Ex]) -> SearchStrategy[Ex]:  # pragma: no cover
+    ...
 
 
 @overload  # noqa: F811
 def one_of(
-    a1: SearchStrategy[Ex], a2: SearchStrategy[T]
-) -> SearchStrategy[Union[Ex, T]]:
-    raise NotImplementedError
+    __a1: SearchStrategy[Ex], __a2: SearchStrategy[T]
+) -> SearchStrategy[Union[Ex, T]]:  # pragma: no cover
+    ...
 
 
 @overload  # noqa: F811
 def one_of(
-    a1: SearchStrategy[Ex], a2: SearchStrategy[T], a3: SearchStrategy[T3]
-) -> SearchStrategy[Union[Ex, T, T3]]:
-    raise NotImplementedError
+    __a1: SearchStrategy[Ex], __a2: SearchStrategy[T], __a3: SearchStrategy[T3]
+) -> SearchStrategy[Union[Ex, T, T3]]:  # pragma: no cover
+    ...
 
 
 @overload  # noqa: F811
 def one_of(
-    a1: SearchStrategy[Ex],
-    a2: SearchStrategy[T],
-    a3: SearchStrategy[T3],
-    a4: SearchStrategy[T4],
-) -> SearchStrategy[Union[Ex, T, T3, T4]]:
-    raise NotImplementedError
+    __a1: SearchStrategy[Ex],
+    __a2: SearchStrategy[T],
+    __a3: SearchStrategy[T3],
+    __a4: SearchStrategy[T4],
+) -> SearchStrategy[Union[Ex, T, T3, T4]]:  # pragma: no cover
+    ...
 
 
 @overload  # noqa: F811
 def one_of(
-    a1: SearchStrategy[Ex],
-    a2: SearchStrategy[T],
-    a3: SearchStrategy[T3],
-    a4: SearchStrategy[T4],
-    a5: SearchStrategy[T5],
-) -> SearchStrategy[Union[Ex, T, T3, T4, T5]]:
-    raise NotImplementedError
+    __a1: SearchStrategy[Ex],
+    __a2: SearchStrategy[T],
+    __a3: SearchStrategy[T3],
+    __a4: SearchStrategy[T4],
+    __a5: SearchStrategy[T5],
+) -> SearchStrategy[Union[Ex, T, T3, T4, T5]]:  # pragma: no cover
+    ...
 
 
 @overload  # noqa: F811
-def one_of(*args: SearchStrategy[Any]) -> SearchStrategy[Any]:
-    raise NotImplementedError
+def one_of(*args: SearchStrategy[Any]) -> SearchStrategy[Any]:  # pragma: no cover
+    ...
 
 
 @defines_strategy(never_lazy=True)
-def one_of(*args):  # noqa: F811
+def one_of(
+    *args: Union[Sequence[SearchStrategy[Any]], SearchStrategy[Any]]
+) -> SearchStrategy[Any]:  # noqa: F811
     # Mypy workaround alert:  Any is too loose above; the return parameter
     # should be the union of the input parameters.  Unfortunately, Mypy <=0.600
     # raises errors due to incompatible inputs instead.  See #1270 for links.
@@ -778,7 +789,7 @@ class MappedSearchStrategy(SearchStrategy):
     """
 
     def __init__(self, strategy, pack=None):
-        SearchStrategy.__init__(self)
+        super().__init__()
         self.mapped_strategy = strategy
         if pack is not None:
             self.pack = pack
@@ -806,17 +817,22 @@ class MappedSearchStrategy(SearchStrategy):
         raise NotImplementedError(f"{self.__class__.__name__}.pack()")
 
     def do_draw(self, data: ConjectureData) -> Ex:
-        for _ in range(3):
-            i = data.index
-            try:
-                data.start_example(MAPPED_SEARCH_STRATEGY_DO_DRAW_LABEL)
-                result = self.pack(data.draw(self.mapped_strategy))
-                data.stop_example()
-                return result
-            except UnsatisfiedAssumption:
-                data.stop_example(discard=True)
-                if data.index == i:
-                    raise
+        with warnings.catch_warnings():
+            if isinstance(self.pack, type) and issubclass(
+                self.pack, (abc.Mapping, abc.Set)
+            ):
+                warnings.simplefilter("ignore", BytesWarning)
+            for _ in range(3):
+                i = data.index
+                try:
+                    data.start_example(MAPPED_SEARCH_STRATEGY_DO_DRAW_LABEL)
+                    result = self.pack(data.draw(self.mapped_strategy))
+                    data.stop_example()
+                    return result
+                except UnsatisfiedAssumption:
+                    data.stop_example(discard=True)
+                    if data.index == i:
+                        raise
         raise UnsatisfiedAssumption()
 
     @property
@@ -857,7 +873,7 @@ class FilteredStrategy(SearchStrategy):
             self._cached_repr = "{!r}{}".format(
                 self.filtered_strategy,
                 "".join(
-                    ".filter(%s)" % get_pretty_function_description(cond)
+                    f".filter({get_pretty_function_description(cond)})"
                     for cond in self.flat_conditions
                 ),
             )
